from datetime import datetime
import re
from html.parser import HTMLParser

from typing import Dict, List


def _extract_image_url(html_string: str) -> str:
    pattern = '<img[^>]+src="([^">]+)"'
    match = re.search(pattern, html_string)
    if match:
        return match.group(1)
    else:
        raise ValueError("Could not extract the image url from html string")


def _extract_text_from_html(html_content: str) -> str:
    text = []

    class TextExtractor(HTMLParser):
        def handle_data(self, data):
            text.append(data)

    parser = TextExtractor()
    parser.feed(html_content)
    return ''.join(text).strip()


class Feed:

    def __init__(self, rss_item: Dict[str, any]):
        self._short_id = rss_item["id"].split("=")[1]
        self._id = rss_item["id"]
        self._link = rss_item["link"]
        self._title = rss_item["title"]
        self._description = _extract_text_from_html(rss_item["description"])
        self._published = datetime.strptime(rss_item["published"], '%a, %d %b %Y %H:%M:%S %z')
        self._image_url = _extract_image_url(rss_item["summary"])
        self._authors = [x["name"] for x in rss_item["authors"]]
        self._tags = [x["term"] for x in rss_item["tags"]]
        self._raw_rss_item = rss_item.copy()

    @property
    def short_id(self):
        return self._short_id

    @property
    def id(self):
        return self._id

    @property
    def link(self):
        return self._link

    @property
    def title(self):
        return self._title

    @property
    def description(self):
        return self._description

    @property
    def published(self):
        return self._published

    @property
    def image_url(self):
        return self._image_url

    @property
    def authors(self):
        return self._authors

    @property
    def tags(self):
        return self._tags

    @property
    def raw_rss_item(self):
        return self._raw_rss_item

    def __iter__(self):
        yield 'id', self._id
        yield 'link', self._link
        yield 'title', self._title
        yield 'description', self._description
        yield 'published', self._published
        yield 'image_url', self._image_url

    def to_dict(self):
        return dict(self)

    @staticmethod
    def load_feed(items: List[Dict[str, str]]) -> List['Feed']:
        feed_items: List['Feed'] = []
        for item in items:
            feed_items.append(Feed(item))
        return feed_items



from datetime import datetime
from os import PathLike, path, makedirs
from typing import Union, List

import feedparser

from feed import Feed

_last_feed_date_file = "/a2y_last_feed_date.txt"


class RSSManager:
    FEED_COMPLETE = "https://www.anime2you.de/feed/"
    FEED_ARTICLES = "https://www.anime2you.de/news/feed/"
    FEED_STREAMS = "https://www.anime2you.de/video-on-demand/feed/"

    def __init__(self, feed_type: str = "https://www.anime2you.de/feed/", use_cache: bool = False, cache_path: Union[
        str, PathLike] = "./cache"):
        """
        :param feed_type:
        :param use_cache:
        :param cache_path:
        :param custom_functions:
        """
        assert feed_type in [
            RSSManager.FEED_COMPLETE,
            RSSManager.FEED_ARTICLES,
            RSSManager.FEED_STREAMS
        ], "Invalid feed type"

        self._feed_url = feed_type

        self._use_cache = use_cache
        self._cache_path = cache_path
        if use_cache:
            if not path.exists(self._cache_path):
                makedirs(self._cache_path)

    def get_feed(self) -> List[Feed]:
        """
        Get the current feed with all entries
        :return:
        """
        parsed = feedparser.parse(self._feed_url)
        feeds = Feed.load_feed(parsed.entries)
        if self.use_cache:
            self.set_cache_feed_published(feeds[0].published)
        return feeds

    def get_new_feed(self):
        """
        Get all new feed entries
        :return:
        """
        if not self.use_cache:
            return self.get_feed()

        newest_feed_published = self.get_cache_feed_published()
        feeds = self.get_feed()
        for feed in feeds.copy():
            if feed.published <= newest_feed_published:
                feeds.remove(feed)
        return feeds

    def set_cache_feed_published(self, date: datetime):
        """
        Need a datetime
        :param date:
        :return:
        """
        with open(self._cache_path + _last_feed_date_file, mode='w') as cf:
            cf.write(date.isoformat())

    def get_cache_feed_published(self) -> datetime:
        """
        Returns a valid datetime
        :return:
        """
        try:
            with open(self._cache_path + _last_feed_date_file, mode='r') as cf:
                return datetime.fromisoformat(cf.read())
        except FileNotFoundError:
            return datetime.fromisoformat("2000-01-01T00:00:00+00:00")

    @property
    def feed_url(self) -> str:
        return self._feed_url

    @property
    def use_cache(self) -> bool:
        return self._use_cache

    @property
    def cache_path(self) -> str:
        return self._cache_path



import pathlib
from setuptools import setup

HERE = pathlib.Path(__file__).parent
README = (HERE / "README.md").read_text()

VERSION = '1.0.0'

setup(
    name='Anime2You',
    long_description=README,
    long_description_content_type="text/markdown",
    version=VERSION,
    packages=[
        "anime2you",
    ],
    url='https://github.com/princessmiku/anime2you',
    license='MIT',
    author='Miku',
    author_email='',
    description='Unofficial RSSClient for Anime2You',
    keywords=['anime2you', 'rss', 'news', 'anime', 'rssclient', 'german'],
    python_requires='>=3.7.0',
    install_requires=[
        'feedparser>=6.0.11',
    ],
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
)
